/**
  WorldObject.java

 Copyright (C) 1998 Geometrek and 2005 in part by Russ Kinter

 *  This file is part of SET Matrix 1.
 *
 *  SET Matrix 1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SET Matrix 1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package matrix.deck.eai;

import matrix.vrml.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.CtrlEvent;
import java.io.*;
import java.util.*;
import vrml.external.*; 
import vrml.external.field.*;
import vrml.external.exception.*;

/**
 *  manages the NetworkState nodes of a world
 *
 *  @version 1.3
 */
public class WorldObject implements RealTimeConsumer {

	private EventInMFString EISMT = null;
       /**
     * New 11-12-05 EventIn for WorldReceiveChat class. Given to the UserAvatar and then to the Beta class. Seemingly unnecessarily long process stops prevents crashes in Cosmoplayer.
     */
    private EventInMFString yak = null;
      /**
     * New 11-12-05 WorldSendAv_Name_Url class created on finding of instance of "MatrixAvatar_Name_Url"  Proto. Sends user name and avatar url to world.
     */
    private WorldSendAv_Name_Url WSANU = null;
   /**
     * New 11-12-05 tf is made true when an instance of Matrix Proto "WorldSendAv_Name_Url" is found in
     * the screen graph. This allows the VRML strings to be sent after the whole scene graph has been read. 
     */
    private boolean tf = false;
    /**
     * New 11-12-05 WorldRecieveChat class allows VRML world to receive text from user and network. Instanced when instance of Matrix Proto "MatrixReceiveChat" is found in scene graph.
     */
    private WorldReceiveChat WRC = null;
     /**
     * New 11-12-05 WorldSendChat class allows VRML world to send text from the world to the network. Instanced when instance of Matrix Proto "MatrixSendChat" is found in scene graph.
     */
    private WorldSendChat WSC = null;
	private boolean bObject = false, bOR = true;
    /**
     * New 11-12-05 vn is VRML browser viewer number to establish workarounds for plugin foibles.
     */
    private int vn = 0;
    private String name;
    /**
     *  Sets top most Viewpoint in stack as true
     */
    private PrimeRoute PR = null;
    private EventInSFInt32 setNav = null;
    private EventInSFFloat setSpeed = null;
    private EventInSFBool binder = null;
	private navObserver NavObs = null;
    private speedObserver SpeedObs = null;
    public boolean bAvatar = false;
     /**
     * Holds ProximitySensor for distributing UserAvatar location  over network.
     */
    private Node proxy = null;
    //private Node PM = null;
     /**
     * New 11-12-05 foo2 is for instance of Matrix Proto "MatrixLights" Allows for delivering a boolean to VRML world for certain tasks like turning off lights when leaving to deal with memory leaks in certain VRML browsers ( Contact 4.4 and WorldView mainly )
     */
    private Node  foo2 = null; 
    private Network postOffice;
      /**
     *  Holds Route classes created from SharedEvent Protos instances found.
     */
    private Vector states = new Vector();
   /**
     *  Distance needed on Y axis to correctly/realistically position UserAvatar on ground
     */
    private float offset;
      /**
     *  Holds Gate classes created from MatrixGate Protos instances found.
     */
    private Hashtable ctrlevents = new Hashtable();
     
    private Vector gates = new Vector();
	private Vector anchors = new Vector();
    private Beta deck;
    /**
     * builds a new WorldObject. It parses the loaded world nodes and 
     * builds all the necessary Route objects, initializes the world.
     * New 11-12-05 Besides the looking for instances of new Protos, 
     * ( MatrixLights, MatrixSendChat, MatrixReceiveChat, MatrixAvatar_Name_Url )
     * the constructor sends a signal when it is finished by calling deck.world_loaded
     *
     * @param object the loaded object, containing the name and a Transform
     * @param browser reference to the VRML browser
     * @param postOffice reference to the Network object
     * @param beta reference to the Applet class for Matrix_Gates
     * @param vpIndex index to which Viewpoint to bind
     *
     
     */
  public class navObserver implements EventOutObserver {
  private boolean bLoaded = false;
  private EventOutMFString navChanged = null;
 
	    public navObserver( EventOutMFString nav_changed ) throws Exception {
	 navChanged = nav_changed;
if(!bLoaded){ navChanged.advise(this, new Integer(17) );
   bLoaded = true;}
     }

  public void callback(EventOut value, double time, Object data) {
 try {
	    if( value == null ) {
		System.err.println("Got null value from navChanged");
		return;
	    }
	    if(!(value instanceof EventOut)) {
		System.err.println("value of class "+value.getClass().getName()+" from navChanged");
		return;
	    }
 if( value.getType()==FieldTypes.MFSTRING ) {
    postOffice.setNavChange( navChanged.getValue()  );
    }
   else 
    return;
    }
  catch(Exception e) {
System.err.println("navObserver callback: "+e);
	}
  }
  public void delete(){                              
if(navChanged!=null&&bLoaded){ navChanged.unadvise(this);	}
    }

  }

 public class speedObserver implements EventOutObserver {
  private boolean bLoaded = false;
  private EventOutSFFloat floatChanged = null;
 
	    public speedObserver( EventOutSFFloat float_changed ) throws Exception {
	 floatChanged = float_changed;
if(!bLoaded){ floatChanged.advise(this, new Integer(6) );
   bLoaded = true;}
     }

public void callback(EventOut value, double time, Object data) {
try {
	    if( value == null ) {
		System.err.println("Got null value from floatChanged");
		return;
	    }
	    if(!(value instanceof EventOut)) {
		System.err.println("value of class "+value.getClass().getName()+" from floatChanged");
		return;
	    }	
	if( value.getType()==FieldTypes.SFFLOAT ) {
    postOffice.setSpeedChange( floatChanged.getValue()  );
    }
    else 
   return;
   }
 catch(Exception e) {
System.err.println("speedObserver callback: "+e);
	}
  }

  public float getSpeed(){
  float ft = floatChanged.getValue();
  return ft;
    }

  public void delete(){                              
if(floatChanged!=null&&bLoaded){ floatChanged.unadvise(this);	}
    }

  }

    public WorldObject( SO1 object, Browser browser, Network postOffice, Beta beta, int vpIndex, int VIN, long slp, String localurl ) throws ConsumerException {
    deck = beta;
    Node [] testNode = null;
	name = object.getName();
	this.postOffice = postOffice;
	// get nodes
	Node xform = object.getTransform();
	Node[] nodes = ((EventOutMFNode)xform.getEventOut("children_changed")).getValue();	
	if( nodes == null ){
	    System.out.println( name +" nodes not loaded !");
	    return;
	}
	
	offset = (float)2.0;
 vn = VIN;
	boolean viewBound = false, fogBound = false, backBound = false, infoBound = false;
 WSC = new WorldSendChat();
 WSANU = new WorldSendAv_Name_Url();
 WRC = null;
 Node[] trans = null;
	Node infoNode = null;
	try {
	trans = ((EventOutMFNode)(object).getTransform().getEventOut("children_changed")).getValue();
	}
    catch(InvalidEventOutException f){ }
	catch(Exception e){
	    System.out.println("PANIC : can't get world nodes "+e);
	    throw new ConsumerException("Error accessing loaded world !");
	}
	// find that vital Group node
	for( int i = 0; i < trans.length; i ++ ) {
	    if( trans[i].getType().equals("Group")) {
		infoNode = trans[i];
		break;
	    }
	}
	if( infoNode == null ){
	    System.err.println("PANIC : Couldn't get Info node !");
	    throw new ConsumerException("Loaded world format is wrong !");
	} else {
	    // parsing Group node
	    trans = ((EventOutMFNode)infoNode.getEventOut("children")).getValue();
	    // if vpIndex is set for the load we subtract 2 to get to -1 by the
	    // time we hit the right viewpoint
	    if( vpIndex > 0 ){
		vpIndex -= 2;
	    }
    boolean wlk = false;
	    for( int i = 0 ; i < trans.length; i += 1 ) {
          System.out.println("for what: " + i );
		  if(!beta.bLogOut){
            if(i!=0&&i%19==0){ beta.onHeartBeat(); 
      beta.worldLoadWarning("World still loading...please wait!" ); 
          }
	   System.out.println("WO types: " + trans[i].getType() + " i: " + i );
      if(trans[i].getType().equals("Walker")){
		    proxy = trans[i];
    	   	    try{
 setNav =  ((EventInSFInt32)proxy.getEventIn("set_navigation"));
 setSpeed = ((EventInSFFloat)proxy.getEventIn("set_speed"));
   }
  catch(InvalidEventInException f){ System.out.println("WorldObject proxy: set_navigation or set_speed error! Please check Walker Proto for error! " + f ); break; }
    try {
  String [] Mfs = null;
 int s  = ((EventOutMFString)proxy.getEventOut("type")).getSize(); 
  if(s==0){ 
  Mfs = new String [] {"WALK","FLY","EXAMINE","FREEFLY","LOOKAT"};
   ((EventInMFString)proxy.getEventIn("type")).setValue(Mfs);
   }
  else {
   Mfs	= new String [s];
  Mfs = ((EventOutMFString)proxy.getEventOut("type")).getValue();
    }
  if(Mfs!=null){postOffice.setNavChange(Mfs);}
 }
catch(InvalidVrmlException d){System.out.println("WorldObject navigation type error: " + d); break; }
catch(InvalidEventOutException f){System.out.println("WorldObject navigation type error: " + f); break; }
catch(InvalidEventInException g){System.out.println("WorldObject navigation type error: " + g); break; }
catch(Exception e){System.out.println("WorldObject navigation type error: " + e); break; } 
		   try {
 NavObs = new navObserver( (EventOutMFString)proxy.getEventOut("type") );
 }
catch(Exception e){System.out.println("WorldObject navigation_changed error: " + e); break; }
	 try {
 SpeedObs = new speedObserver( (EventOutSFFloat)proxy.getEventOut("set_speed") );
 }
catch(Exception e){System.out.println("WorldObject set_speed error: " + e); break; }
 if( infoBound == false ){
            try {
			((EventInSFBool)trans[i].getEventIn("set_navbind")).setValue(true);
            }
            catch(InvalidEventInException f){ System.out.println("WorldObject proxy: set_navbind error! " + f); break; } 
            try{
			float[] help = ((EventOutMFFloat)trans[i].getEventOut("avatarSize")).getValue();
			offset = help[1];
            }
            catch(InvalidEventInException f){ System.out.println("WorldObject proxy: Avatar Size error! " + f); break; }
			infoBound = true;
           System.out.println("Walker found");
             }
       }
      else 
      if(trans[i].getType().startsWith("Network")){
		   Route route = new Route( trans[i], postOffice, vn, slp );
		    route.setName( name+"_"+route.getName());
		    states.addElement( route );
		    postOffice.addConsumer( route );
		  System.out.println("Network found");
        }
	  else if( trans[i].getType().startsWith("Ctrl")){
          if(trans[i].getType().indexOf("Network")!=-1){
		    CtrlRoute CR = new CtrlRoute( trans[i], postOffice, vn, 400, deck.set_AvatarName() );
		    CR.setName( name+"_"+CR.getName());
		    states.addElement( CR );
		    postOffice.addConsumer( CR );
		   addControlEvent( (CtrlEvent)CR );
    	   System.out.println("ControlRoute found");
         }
         else {
           CtrlTime  CT = new CtrlTime( trans[i], postOffice, deck.set_AvatarName() );
			 addControlEvent( (CtrlEvent)CT );
         }
		}
      else 
    	if ( trans[i].getType().equals("Matrix_Gate")){
      Gate gate = new Gate( trans[i], beta, vn );
    gates.addElement(gate);
		System.out.println("Gate found");
		}
       else 
	   		if ( trans[i].getType().equals("Matrix_Text")){
         EISMT = (EventInMFString)trans[i].getEventIn("set_matrixText" );
        // boolean tf  = ((EventOutSFBool)trans[i].getEventOut("bDefault")).getValue();
	          beta.handleMatrixText();
    	 	System.out.println("Matrix Text found");
		   }
	    else 
    	if ( trans[i].getType().equals("Matrix_Anchor")){
      IAnchor ianchor = new IAnchor( trans[i], localurl );
      anchors.addElement(ianchor);
		System.out.println("Anchor found");
		}
	   else 
    if (trans[i].getType().equals("MatrixSendChat")){
     WSC  = new WorldSendChat(beta, trans[i] ); 
	System.out.println("MatrixSendChat found");
      }
     else 
    if (trans[i].getType().equals("MatrixReceiveChat")){
 try {
  yak = (EventInMFString)trans[i].getEventIn("set_ChatIn");
 }
catch(NullPointerException e){ 
System.out.println("MatrixReceiveChat: set_ChatIn not found!" + e); 
 break;
 }
catch(InvalidEventOutException ieoe ) {
 System.out.println("MatrixReceiveChat: set_ChatIn not found!" + ieoe);
break; 
}
 if(yak!=null){
  WRC = new WorldReceiveChat( postOffice );
  WRC.set_yakker( yak );
  System.out.println("MatrixReceiveChat found");
  }
  
}
  else 
  if (trans[i].getType().equals("MatrixLights")){
   foo2 = trans[i];
  System.out.println("Matrix Lights found");
  }
  else 
  if (trans[i].getType().equals("MatrixAvatar_Name_Url")){
    WSANU.setWSANU( beta.set_AvatarName(), beta.set_AvatarUrl(), trans[i]  );
    tf = true;
   System.out.println("MatrixAvatar_Name_Url found");
 }
   else 
    if(trans[i].getType().equals("PrimeMover")){ 
    Node PM = trans[i];
    bObject = true;
	PR = new PrimeRoute( PM, postOffice, vn, slp, beta.set_AvatarName() );
		    PR.setName( name+"_"+PR.getName());
		    states.addElement( PR );
		    postOffice.addConsumer( PR );
		System.out.println("PrimeMover found");
   }
		else if( trans[i].getType().equals("Viewpoint")) {
				 System.out.println("Viewpoint found");
		    if( viewBound == false ){
			if( vpIndex == -1 ){
        binder = ((EventInSFBool)trans[i].getEventIn("set_bind"));
       binder.setValue(true);
			    viewBound = true;
			} else {
			    vpIndex --;
			}
		    }
		}
		else if( trans[i].getType().equals("Fog")) {
					 System.out.println("Fog found");
		    if( fogBound == false ){
			((EventInSFBool)trans[i].getEventIn("set_bind")).setValue(true);
			fogBound = true;
		    }
		}
		else if( trans[i].getType().equals("Background")) {
					   System.out.println("Background found");
		    if( backBound == false ){
			((EventInSFBool)trans[i].getEventIn("set_bind")).setValue(true);
			backBound = true;
		    }
		}
		 }
    else {   System.out.println("for problem");      
     break;
          }
         }
	    } 
   if(tf==true){ WSANU.sendName_Url();}
   if(proxy!=null){
    beta.world_loaded();
    }
  if(SpeedObs!=null){ setSpeed( SpeedObs.getSpeed() ); }
     else {
    System.out.println("MISSION CRITICAL WALKER PROTO NOT FOUND!");
    return;
    } 
   }

   
     

    /**
     * returns offset found in the NavigationInfo node.
     */
    public float getOffset(){
	return offset;
    }

  
    

      public void setBinder(){
  if(binder!=null){
  binder.setValue(true); }
 }

     /**
     * returns proxy found in the ROOT group node. Used by
     * UserAvatar to track users position.
     */

    public Node getProxy(){
	return proxy;
    }

      public Node getPrimeMover(){
  Node PM = PR.getPrimeMover();
  return PM;
  }
 
   /**
     * New 11-12-05 getChatIn method returns the EventInMFString for the world receiving chat
     * input. It is called by the UserAvatar class and is placed in the Beta for getting
     * chat from both the network and user. Transfering it from the UserAvatar class keeps
     * Cosmoplayer from crashing...for some unknown reason.
     */
 
     public EventInMFString getChatIn(){
	return yak;
    }

    public String getName(){
	return name;
    }

	  public void objectRoutes( String target ){
  Browser browser = deck.getBrowser();
  Node PM = getPrimeMover();
  if(PM!=null){
  if(bOR==true){
     ((EventInSFVec3f)PM.getEventIn("translation")).setValue( ((EventOutSFVec3f)proxy.getEventOut("translation_changed")).getValue() );
     ((EventInSFRotation)PM.getEventIn("rotation")).setValue( ((EventOutSFRotation)proxy.getEventOut("rotation_changed")).getValue() );
   browser.addRoute( proxy, "rotation_changed", PM, "rotation");
   browser.addRoute( proxy, "translation_changed", PM, "translation");
   ((EventInSFInt32)PM.getEventIn("whichChoice")).setValue(0);
  PR.userView("WAKE");
  bOR = false;
   }
  else {
   ((EventInSFInt32)PM.getEventIn("whichChoice")).setValue(-1);
   browser.deleteRoute( proxy, "rotation_changed", PM, "rotation");
   browser.deleteRoute( proxy, "translation_changed", PM, "translation");
   PR.userView("WAIT");
     bOR = true;
    }
   }
  }

   public boolean getbAvatar(){
  return bAvatar;
     }

    public void setUserPosition( float[] flt ){
  }

  public void checkPosition( float[] fUP, boolean bUPs ){
  }


       public synchronized void addControlEvent( CtrlEvent ctrlevt ){
	if( ctrlevents.containsKey(ctrlevt.ctrlName())){
	    System.err.println("WorldObject "+ctrlevt.ctrlName()+" exists allready");
	    return;
	}
	ctrlevents.put( ctrlevt.ctrlName(), ctrlevt );
    }

    public synchronized void handleControlEvent( String target, String [] lock ){
		  CtrlEvent ctrlvt = (CtrlEvent) ctrlevents.get(target);
         if(ctrlvt!=null){ ctrlvt.handleLock( lock );}
    }

    public void setWorldMessage( String  line, String name    ){
    if(WRC!=null){ WRC.setYak( line, name ); }
    }

	 public void setMatrixTextMsgs( String [] mtm ){
     if(EISMT!=null){ EISMT.setValue(mtm); }
     }
    
    public void handleMessage( String tag, Field data ) throws ConsumerException {
	throw new ConsumerException("WorldObject should not receive any messages !");
    }

    public void update(){
	Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    ((RealTimeConsumer)list.nextElement()).update();
	}	
    } 

   public void sharedObjectName( String target, int state ){
  if(PR!=null){
   PR.sharedObjectName( target, state );
   }
 }

 public void beamingObjectName( String target, float [] mft, int state ){
	 if(PR!=null){
   PR.beamingObjectName( target, mft, state );
   }
 }

  /**
   * New 11-12-05 reset method Calling VRML EventIns do turn off stuff some before
   * the delete method is called in RealTimeHandler simply seems to work better
   * when changing rooms and disconnecting.
   */


 public void reset(){
 tf = false;
 //if(foo2!=null){ ((EventInSFBool)foo2.getEventIn("set_bool")).setValue(false);}
 //if(proxy!=null){ ((EventInSFBool)proxy.getEventIn("enabled")).setValue(false); }
  } 

    public void delete() throws RuntimeException {
	Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    postOffice.removeConsumer(((RealTimeConsumer)list.nextElement()).getName());
	}
	states.removeAllElements();
  Enumeration list2 = gates.elements();
	while( list2.hasMoreElements()){
	    ((Gate)list2.nextElement()).delete();
	}
 gates.removeAllElements();
 Enumeration list3 = anchors.elements();
	while( list3.hasMoreElements()){
	    ((IAnchor)list3.nextElement()).delete();
	}
 anchors.removeAllElements();
 Enumeration list4 =  ctrlevents.elements();
	while( list4.hasMoreElements()){
	   ((CtrlEvent)list4.nextElement()).ctrlDelete();
	}
 ctrlevents.clear();
  WSANU.delete();
  WSC.delete();
   binder = null;
 yak = null;
  proxy = foo2 = null;
 if(WRC!=null){ WRC.delete();}
 if(NavObs!=null){ NavObs.delete();}
 if(SpeedObs!=null){ SpeedObs.delete(); }
 //whichChoice = null;
    }

  /**
     * New 11-12-05 beamto is here because it is a member of the RealTimeConsumer interface
     * It doesn't do anything except stop an error
     */

   public void beamto( String target ){
}

   public void userView( String target ){
 }
  

  public void setNavigation( int num ){
 if(setNav!=null){	setNav.setValue( num );	}
 }

 public synchronized void setSpeed( float ft ){
   if(setSpeed!=null){ setSpeed.setValue( ft ); }
   }

}
