package matrix.deck.eai;

import matrix.vrml.*;
import matrix.deck.*;
import java.io.*;
import java.util.*;
import vrml.external.*; 
import vrml.external.field.*;
import vrml.external.exception.*;

/**
 * New 11-12-05 WorldReceiveChat sends chat messages from world to network chat via the Beta class.
 * Instanced by WorldObject class.
 * @version 0.1    
 * @author Russ Kinter
 */

public class WorldSendChat implements EventOutObserver, Runnable {
    /**
     *  New 11-12-05 chat holds the eventOut of Node wsc ( WorldSendChat ) called "to_chat" 
     *  It is given an EventObserver to watch for new Strings
     */
  private EventOutMFString chat;
      /**
     *  New 11-12-05 Beta deck to which the Strings are sent for output over the network
     */
  private Beta deck;
     /**
     *  New 11-12-05 wsc holds the instance of the Proto WorldSendChat
     */
  private Node wsc;
    /**
     *  New 11-12-05 runner controls the rate of how many strings go to Beta deck and the network.
     */
  private Thread runner = null;
    /**
     *  New 11-12-05 Vector vec is a buffer storing strings before the Thread runner sends them.
     */
  private Vector vec = new Vector();


  public  WorldSendChat(Beta beta, Node WSC ){  
  deck = beta; 
  wsc = WSC; 
  chat = (EventOutMFString)wsc.getEventOut("to_chat");
  chat.advise(this, new Integer(15) );
  runner = new Thread( this,"Yak_To_Chat" );
  runner.start();
}

  public WorldSendChat(){
}

  private void holdChat() throws InterruptedException {
    synchronized(vec) { 
     vec.wait();
    }
   }

   /**
     *  New 11-12-05 EventOut Observer that loads new incoming Strings from the VRML World into the Vector "vec"
     *  which in turn wakes up the Thread Runner to send to Beta Deck in 1 second interegals
     */

public void callback(EventOut value, double time, Object data) {
 int b = 0;
	try {
	    if( value == null ) {
		System.err.println("Got null value from ");
		return;
	    }
	    if(!(value instanceof EventOut)) {
		System.err.println("value of class "+value.getClass().getName() );
		return;
	    }
 if(value.getType()==FieldTypes.MFSTRING){
     b = ((EventOutMFString)chat).getValue().length;
     String[] st = new String[b];
     st = ((EventOutMFString)chat).getValue();
  synchronized(vec) { 
     for (int c = 0; c<b; c++){ 
      vec.insertElementAt(st[c],c); } 
       vec.notify();
       }
      }
	    }
	catch(Exception e) {
	    System.err.println("Error converting msg from world"+"\n\t" +e);
	}
    }


  /**
     *  New 11-12-05 sendChat is called by run once a second ( wait(1000) if the Vector "vec" still
     *  contains Strings. If it doesn't, it goes to sleep ( wait() ) till waken up by callback - the eventOut observer 
     */

 private void sendChat() throws InterruptedException {
    synchronized(vec) { 
       if(!vec.isEmpty()){
       String st2 = vec.firstElement().toString();
       deck.handleChat2( st2, null );
       vec.removeElementAt(0);
       vec.wait(1000);
        }
    else { 
      vec.wait();
      }
    }
  }
  
 /**
     *  New 11-12-05 When run first starts it goes into wait by calling holdChat
     */

 public void run(){
    try { 
     holdChat();
      }
    catch(InterruptedException e){}
  while( Thread.currentThread() == runner ) {
 try {
    sendChat();
   } 
 catch(InterruptedException e){}
 
  }
}

    /**
     *  New 11-12-05 clean up at either disconnect or room change. 
     *  Notifying the Vector being used as a buffer at clean up was used in the original
     *  DeepMatrix (Connection inner class MessageBuffer ) and therefore adopted here as well. 
     */

  public void delete(){
  if(chat!=null){ chat.unadvise(this);}
   synchronized(vec){ vec.removeAllElements(); vec.notify() ;}
   runner = null; chat = null; wsc = null;
   } 
}