/**
  Avatar.java

 Copyright (C) 1998 Geometrek	and 2009 by R. Russell Kinter
 
 *  This file is part of DeepMatrixIP9.
 *
 *  DeepMatrixIP9 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix IP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.deck.eai2;

import matrix.vrml.*;
import matrix.deck.RealTimeConsumer;
import matrix.deck.ConsumerException;
import matrix.deck.Network;
import matrix.deck.CtrlEvent;
import matrix.deck.NetworkListener;
import java.io.*;
import java.util.*;
import vrml.eai.*; 
import vrml.eai.field.*;


/**
 *  manages a single avatar
 *
 *  @version 1.3
 *
 * New 11-12-05 the gesture list is now also used to list Avatar options 
 * (NG_chat_text,NG_humanoid,NG_no_lod) "NG_" prefix is for the UserAvatar
 * so the option does not show up as a gesture. 
 * NG_chat_text means the avatar is setup for chat text display.
 * NG_humanoid the avatar has Deep Matrix's humanoid animation methods: "inMotionBehaviour" 
 * and "notInMotionBehaviour"
 * NG_no_lod means the avatar does not want the built in LOD
 *
 * drone_av_finished( name, false ) is called from Beta if the avatar is either finished or throws an error.
 */
public class Avatar implements RealTimeConsumer {
    /**
    * New 11-12-05 AvatarTextTalk class for delivering chat text strings to drone avatar display.
    */
    private AvatarTextTalk texttalk = null;
  
    private int iMove = -2, iMove2 = 0, iMove3 = 0, iMCt = 2, iMCt2 = 0, iAv1 = 0, iAv2 = -1;

	private float [] fUserPosition = new float [] { 0.0f, 0.0f, 0.0f };
    
	private float [] fCurrPos = new float [] { 0.0f, 0.0f, 0.0f };
    private int iTct = 0;

    public boolean bAvatar = false, bMove = false, bHumanoid = false; 
   
    private boolean bdmStudioAvatar = false;

    private boolean bUserNear1 = false, bUserNear2 = true, bDelete = true;

    private int iUserNear1 = -2, iUserNear2 = 0, iCheckPos = 0; 

    private EventInSFInt32 setProxy = null;
    /**
    * New 11-12-05 String variables for keep track of avatar features
    */
    private String talk = "nah",human = "hum", lod = "bah", inMotionBehaviour = null, notInMotionBehaviour = null; 
     /**
    * New 11-12-05 Lodst is for built-in drone Avatar Level of Detail to be created to VRML from String.
    */
    private String Lodst = ( "Group { " +   
             "children [ " + 
             " LOD { " + 
             "range [30 80] " +   
             " level [ " +  
             "Transform { " +
						       "rotation 0 1 0 3.141 " +
                               "addChildren IS addLOD_Children " +
            				   "removeChildren IS removeLOD_Children " +
						       "children IS lod_children " +  
             "  }  " +   
             "Transform {  " +  
             "translation	0 0 0 " + 
             "children [	Shape {  " +
             "appearance	NULL  " + 
             "geometry	 IndexedFaceSet {  " + 
             "coord	Coordinate {  " + 
	            " point	[ -0.25 1.75 0.125, "  +
		  " -0.25 0.0 0.125,"  +
		  " 0.25 1.75 0.125, "  +
		  " 0.25 0.0 0.125, "  +
		  " 0.25 1.75 -0.125, "  +
		  " 0.25 0.0 -0.125, "  +
		  " -0.25 1.75 -0.125, "  +
		  " -0.25 0.0 -0.125 ] "  +
     "  } "  +
     " coordIndex	[ 0, 1, 3, 2, -1, 4, 5, 7, "  +
	       "  6, -1, 6, 7, 1, 0, -1, 2, "  +
	       "  3, 5, 4, -1, 6, 0, 2, 4, "  +
	       "  -1, 1, 7, 5, 3, -1 ] "  +
             "color Color { color [ .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3, .3 .3 .3,  .3 .3 .3 ]}  " +  
             "}}]}  " + 
             "Group { } " + 
             "]}]} " );

	/**
    * New 11-12-05 field2 and field2 hold the talker_text and set_name eventIns of a drone avatar if found.
    */
    private EventInSFString set_name = null;
   /**
   /**
    * New 11-12-05 field2 and field2 hold the talker_text and set_name eventIns of a drone avatar if found.
    */
    private EventIn field2, field3;
   /**
    * New 11-12-05 field holds Avatar gestures MFString
    */
    private EventOut field;
   /**
    * New 11-12-05 Beta is added as variable for beamto
    */
    private Beta avdeck;
     /**
    * New 11-12-05 a is used as a boolean to determine if chat text is to be displayed.
    */
    private int a = 0;
     /**
    * New 11-12-05 collects position and rotation together for beam to information
    */ 
    private float [] beam = new float[7];
    private float [] fRange = new float [0];
   /**
    * New 11-12-05 position with beam to information
    */ 
    private float [] posflt = new float[3];
   /**
    * New 11-12-05 rotation with beam to information
    */ 
    private float [] rotflt = new float[4];
    private String name;
    private Network postOffice;
	private EventInSFInt32 NameState = null;
     /**
    * New 11-12-05 Node for Chat
    */
    private EventInMFString yak = null;
     /**
    *  Holds the avatar
    */
    private Node avatar = null; 
   /**
    *  Grouping Node
    */
    private Node anchor = null;

    private EventInSFBool [] gestures = null;

    private EventInSFBool stand = null;

	private EventInSFBool move = null;

   /**
    *  holds avatars position.
    */
    private EventInSFVec3f pos;


	private EventInSFInt32 which;
    private EventInSFRotation rot;
    /**
    * New 11-12-05 pos2 EventOuts for beam to coordinate position.
    * Same exposedField as "pos" 
    */
    private EventOutSFVec3f pos2 = null;
   /**
    * New 11-12-05 rot2 EventOuts for beam to coordinate rotation
    * Same exposedField as "rot"  
    */
    private EventOutSFRotation rot2 = null;
    /**
     * New 11-12-05 name_text is for users name display to be create VRML from String
     */
    private String name_text; 
    private String avUrl = "";
    
    private String name2 = "";

   /**
    * holds Routes for found Shared Event Protos
    */
    private Vector states = new Vector();
    private Vector vCtrlevents = new Vector();
    private WorldSendChat WSC = null;
    private WorldSendLocalChat WSLC = null; 
    public boolean bAvOk = true, bObject = false, bname_state = true, bOR = true, bP = true, bO = true;

    /**
     * builds a new Avatar. It parses the loaded nodes and construct the needed scenegraph
     * and gets the events etc. Also builds all the necessary Route objects, if it is
     * a SharedObject.
     *
     * @param object the loaded object, containing the name and a Transform
     * @param browser reference to the VRML browser
     * @param postOffice reference to the Network object
     *
     */

    public Avatar( SO2 object, Browser browser, Network postOffice, Beta deck, int VIN, long slp ) throws RuntimeException {
  if(!deck.bLogOut){
  Node [] Foo = null;
 Node [] xform = null;
 Node [] nodes = null;
 Node [] help3 = null;
 EventOutMFNode nsc = null;
 boolean bGest = false;
 boolean bHAnim = false;
 name = object.getName();
 avUrl = object.getUrl();
 boolean bTest = false;
 EventOut EOfield = null;
 EventIn EIfield = null;
 if(!name.startsWith("_")){
 bObject = true;
 name2 = name;
 }
 else {
 name2 = name.substring(1);
 name_text = ( "Transform { " + 
			"translation 0 1.75 0 " +
			"children [ " + 
             "LOD {  " +
             "range 25 " +                                         
             "level [ " +
              "Group { " +
              "children [ " +
			  "Billboard {   " +
              "axisOfRotation 0 0 0 " +
			  "children [  " +
			  "Shape { geometry Text { string [\"" + name2 + "\"] maxExtent 4  " +
			  "fontStyle FontStyle { family [\"SANS\"] justify \"MIDDLE\" size .25 } } " +
			  "} " +
             "]}]} " +
             "Group {} " +                                          
						       "]}]}  " );
 }
	this.postOffice = postOffice;
 avdeck = deck;
 	// get nodes
		try {
 xform = object.getTransforms2();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransforms(): " + np );
   avdeck.drone_av_finished( name, false );
	    return;}
if(xform[0].getType().equals("Avatar3")){
  	try {
 Foo = object.getTransforms1();
 nodes = ((EventOutMFNode)Foo[0].getEventOut("children_changed")).getValue();
    }
  catch(NullPointerException np){ System.out.println( "object.getTransform(): " + np );
  avdeck.drone_av_finished( name, false );
	    return; 
     }
	// check for SharedObject definition
	if(bObject&&nodes[0].getType().equals("SharedObject")){
	    Node soNode = nodes[0];
 try {
EIfield = null;
bTest = false; 
EIfield = soNode.getEventIn("from_network_chat");
if(EIfield!=null&&EIfield.getType() == BaseField.MFString ){
yak = (EventInMFString)EIfield;
   }
 }
catch(NullPointerException e){ 
 System.out.println( "SharedObject " + name + " from_network_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
   } 
catch(InvalidEventInException f){
System.out.println( "SharedObject doesn't have from_network_chat.");
   }
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " from_network_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " from_network_chat error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
 try {
EOfield = null;
bTest = false; 
EOfield = soNode.getEventOut("to_network_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
 System.out.println( "SharedObject " + name + " to_network_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
   } 
catch(InvalidEventOutException f){
System.out.println( "SharedObject doesn't have to_network_chat.");
   }
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " to_network_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " to_network_chat error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
 try {
EventOutMFString eomfs = (EventOutMFString)EOfield;
WSC  = new WorldSendChat( deck, eomfs );
System.out.println( "SharedObject " + name + " has to_network_chat.");
  }
catch(NullPointerException e){ 
 System.out.println( "SharedObject " + name + " to_network_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
 System.out.println( "SharedObject " + name + " to_network_chat error: " + f);
    avdeck.drone_av_finished( name, false );
    return;
 
}
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " to_network_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " to_network_chat error: " + h);
    avdeck.drone_av_finished( name, false ); 
    return;
 }
}
try {
EOfield = null;
bTest = false; 
EOfield = soNode.getEventOut("to_local_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
System.out.println( "SharedObject " + name + " to_local_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "SharedObject doesn't have  to_local_chat.");
}
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " to_local_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " to_local_chat error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
try {
EventOutMFString eomfs = (EventOutMFString)EOfield;
WSLC  = new WorldSendLocalChat( deck, eomfs );
System.out.println( "SharedObject has to_local_chat.");
  }
catch(NullPointerException e){ 
	System.out.println( "SharedObject " + name + " to_local_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
   System.out.println( "SharedObject " + name + " to_local_chat error: " + f);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " to_local_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " to_local_chat error: " + h);
    avdeck.drone_av_finished( name, false ); 
    return;
 }
}
try {
EOfield = null;
bTest = false; 
EOfield = soNode.getEventOut("states_changed");
if(EOfield!=null&&EOfield.getType() == BaseField.MFNode ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "SharedObject " + name + " states_changed error: " + e);
  avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "SharedObject doesn't have  states_changed.");
}
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " states_changed error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " states_changed error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
 try {
nsc = (EventOutMFNode)EOfield;
 }
catch(NullPointerException e){ 
System.out.println( "SharedObject " + name + " states_changed error: " + e);
    avdeck.drone_av_finished( name, false ); 
    return;
}
catch(InvalidEventOutException f){ 
System.out.println( "SharedObject " + name + " no states_changed: " + f);
}
catch(InvalidVrmlException g){ 
System.out.println( "SharedObject " + name + " states_changed error: " + g);
    avdeck.drone_av_finished( name, false ); 
    return;
}
catch(Exception h){
System.out.println( "SharedObject " + name + " states_changed error: " + h);
    avdeck.drone_av_finished( name, false ); 
    return;
}
} 
try {
if(nsc!=null){ 
	    Node[] statesVal = nsc.getValue();
	    if( statesVal.length != 0 ){
		for( int i = 0; i < statesVal.length; i += 1 ){
			if(!deck.bLogOut){
		    if( statesVal[i].getType().startsWith("Network")){
			Route route = new Route( statesVal[i], postOffice, VIN, slp );
			route.setName( name+"_"+route.getName());
			postOffice.addConsumer( route );
			states.addElement( route );
		     }
         else 
          if( statesVal[i].getType().startsWith("Ctrl")){
          if(statesVal[i].getType().indexOf("Network")==4){
		    CtrlRoute CR = new CtrlRoute( statesVal[i], postOffice, VIN, 400, deck.set_AvatarName() );
		    CR.setName( name+"#"+CR.getName());
		    states.addElement( CR );
            vCtrlevents.addElement( CR );
		    postOffice.addConsumer( CR );
		   deck.addCtrlEvent( (CtrlEvent)CR );
    	   System.out.println("ControlRoute found");
         }
         else 
		  if(statesVal[i].getType().indexOf("Time")==4){
           CtrlTime  CT = new CtrlTime( statesVal[i], postOffice, deck.set_AvatarName() );
           CT.setCtrlName( name + "#" + CT.ctrlName() );
		   vCtrlevents.addElement( CT );
			deck.addCtrlEvent( (CtrlEvent)CT );
			  System.out.println("CtrlTime found");
         }
          else 
		  if(statesVal[i].getType().indexOf("Remove")==4){
           CtrlRemove CR = new CtrlRemove( statesVal[i], postOffice, deck.set_AvatarName() );
			CR.setCtrlName( name + "#" + CR.ctrlName() );
			vCtrlevents.addElement( CR );
            deck.addCtrlEvent( (CtrlEvent)CR );
          }
		}
           }
	      }
		}
      }
   }
catch(NullPointerException e){ 
System.out.println( "SharedObject " + name + " states_changed error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventInException f){
 System.out.println( "SharedObject " + name + "  states_changed error: " + f);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidEventOutException g){
 System.out.println( "SharedObject " + name + "  states_changed error: " + g);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidNodeException h){
 System.out.println( "SharedObject " + name + "  states_changed error: " + h);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidVrmlException j){
 System.out.println( "SharedObject " + name + " states_changed error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "SharedObject " + name + " states_changed error: " + h);
    avdeck.drone_av_finished( name, false );
    return;
}     // get events
	   // pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	   // rot = (EventInSFRotation)xform[0].getEventIn("rotation");
     } 	
if( nodes[0].getType().equals("Avatar")){
EventOutSFBool eosfb = null;
try {
EOfield = null;
bTest = false;
EOfield = nodes[0].getEventOut("isAvatar");
 if(EOfield!=null){ 
 bTest = true;
   }
 }
catch(NullPointerException e){ 
 System.out.println( "Avatar " + name + " isAvatar error: " + e);
 avdeck.drone_av_finished( name, false );  
 return;
}
catch(InvalidEventOutException f){
System.out.println(name  + " is not a bxx avatar! GOOD continue!");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " isAvatar error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " isAvatar error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){ 
  System.out.println(name  + " is a bxx avatar. Avatar rejected!");
  avdeck.drone_av_finished( name, false );
   return; 
   }
avatar = nodes[0];


try {
EOfield = null;
bTest = false; 
EOfield = nodes[0].getEventOut("states_changed");
if(EOfield!=null&&EOfield.getType() == BaseField.MFNode ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " states_changed error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "Avatar " + name + " doesn't have  states_changed.");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " states_changed error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " states_changed error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
try {
nsc = (EventOutMFNode)nodes[0].getEventOut("states_changed");
 }
catch(NullPointerException e){ 
 System.out.println( "Avatar " + name + " states_changed error: " + e);
 avdeck.drone_av_finished( name, false );
 return;
}
catch(InvalidEventOutException f){ 
 System.out.println( "Avatar " + name + " states_changed error: " + f);
 avdeck.drone_av_finished( name, false );
 return;
}
catch(InvalidVrmlException g){ 
System.out.println( "Avatar " + name + " states_changed error: " + g);
     avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " states_changed error: " + h);
    avdeck.drone_av_finished( name, false );
    return;
  } 
 }
try {
EOfield = null;
bTest = false; 
EOfield = nodes[0].getEventOut("to_network_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " to_network_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "SharedObject doesn't have  to_network_chat.");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " to_network_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " to_network_chat error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
try {
EventOutMFString eomfs = (EventOutMFString)EOfield;
WSC  = new WorldSendChat( deck, eomfs );
  }
catch(NullPointerException e){ 
   System.out.println( "Avatar to_network_chat error: " + e);
    avdeck.drone_av_finished( name, false );
}
catch(InvalidEventOutException f){
   System.out.println( "Avatar to_network_chat error: " + f);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar to_network_chat error: " + j);
    avdeck.drone_av_finished( name, false );	
    return;
 }
catch(Exception h){
System.out.println( "Avatar to_network_chat error: " + h);
    avdeck.drone_av_finished( name, false );
    return;
 }
}
try {
EOfield = null;
bTest = false; 
EOfield = nodes[0].getEventOut("to_local_chat");
if(EOfield!=null&&EOfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " to_local_chat error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventOutException f){
System.out.println( "Avatar doesn't have  to_local_chat.");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " to_local_chat error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " to_local_chat error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
try {
EventOutMFString eomfs = (EventOutMFString)EOfield;
WSLC  = new WorldSendLocalChat( deck, eomfs );
  }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " to_local_chat error: " + e);
	avatar = null;
    avdeck.drone_av_finished( name, false ); 
    return;
}
catch(InvalidEventOutException f){
  System.out.println( "Avatar " + name + " to_local_chat error: " + f);
	avatar = null;
    avdeck.drone_av_finished( name, false ); 
    return;
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " to_local_chat error: " + j);
	avatar = null;
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " to_local_chat error: " + h);
	avatar = null;
    avdeck.drone_av_finished( name, false ); 
    return;
 }
}
try {
EIfield = null;
bTest = false; 
EIfield = nodes[0].getEventIn("talker_text");
if(EIfield!=null&&EIfield.getType() == BaseField.MFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " talker_text error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventInException f){
System.out.println( "Avatar doesn't have  talker_text.");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " talker_text error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " talker_text error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
if(bTest){
 try { 
 yak = (EventInMFString)EIfield;
  }
   catch(InvalidEventInException e){
    System.out.println("Avatar " +name + ": talker_text error " + e);
    avdeck.drone_av_finished( name, false );
    return;
	}
	catch(NullPointerException f){
    System.out.println("Avatar " +name + ": talker_text error " + f);
    avdeck.drone_av_finished( name, false );
    return;
	}
   catch(InvalidVrmlException g){
    System.out.println("Avatar " +name + ": talker_text error " + g);
    avdeck.drone_av_finished( name, false );
    return;
    }
      catch( Exception h){
    System.out.println("Avatar " +name + ": talker_text error " + h);
    avdeck.drone_av_finished( name, false );
    return;
    }
   }
  try {
EIfield = null;
bTest = false; 
EIfield = nodes[0].getEventIn("set_name");
if(EIfield!=null&&EIfield.getType() == BaseField.SFString ){
  bTest = true;
   }
 }
catch(NullPointerException e){ 
  System.out.println( "Avatar " + name + " set_name error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
}
catch(InvalidEventInException f){
System.out.println( "Avatar doesn't have  set_name.");
}
catch(InvalidVrmlException j){
 System.out.println( "Avatar " + name + " set_name error: " + j);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
System.out.println( "Avatar " + name + " set_name error: " + h);
    avdeck.drone_av_finished( name, false );  
    return;
}
 if(bTest){
   try { 
   set_name = (EventInSFString)nodes[0].getEventIn("set_name");
 if(set_name!=null){ set_name.setValue(name2); }
   }
   catch(InvalidEventInException e){
    System.out.println("Avatar " + name + " set_name error: " + e);
    avdeck.drone_av_finished( name, false );
    return;
	}
	catch(NullPointerException f){
     System.out.println("Avatar " + name + " set_name error: " + f);
     avdeck.drone_av_finished( name, false );
     return;
	}
   	catch(InvalidVrmlException g){
     System.out.println("Avatar " + name + " set_name error: " + g);
     avdeck.drone_av_finished( name, false );
     return;
	}
   	catch( Exception h){
     System.out.println("Avatar " + name + " set_name error: " + h);
     avdeck.drone_av_finished( name, false );
     return;
	}
  }
try {
EOfield = null;
bTest = false;
EOfield = nodes[0].getEventOut("gestures");
if(EOfield!=null&&EOfield.getType()==BaseField.MFString){
 bTest = true;
 }
}
  catch(InvalidEventOutException e){
     System.out.println("Avatar " + name+ " does not have gestures!");
	}
  catch(NullPointerException f){ 
	System.out.println("Avatar " + name + " gestures errors: " + f );
    avdeck.drone_av_finished( name, false );
    return;
     }
  catch(InvalidVrmlException g){ 
    System.out.println("Avatar " + name + " gestures errors: " + g );
    avdeck.drone_av_finished( name, false );
    return; }
  catch(Exception h){ 
    System.out.println("Avatar " + name+" gestures errors: " + h );
    avdeck.drone_av_finished( name, false );
    return; }
 if(bTest){
  try{
  int b = (((EventOutMFString)EOfield).getValue()).length;
  int d = b;
   String all [] = new String[b];
   all = (((EventOutMFString)EOfield).getValue());
     for (int c = 0; c<b; c += 1){ 
  if( all[c].equals("NG_no_lod")){ lod = all[c]; d -= 1; }
 if( all[c].equals("NG_chat_text")){talk = all[c]; d -= 1; }
 if( all[c].equals("NG_humanoid")){human = all[c]; d -= 1; }
   }
  gestures = new EventInSFBool [d];
	}
catch(InvalidEventOutException e){
     System.out.println("Avatar " + name + " gesture string error: " + e);
     avdeck.drone_av_finished( name, false ); 
    return;
	}
	catch(NullPointerException f){ 
    System.out.println("Avatar " + name + " gesture string error: " + f );
    avdeck.drone_av_finished( name, false );
    return; }
 catch(InvalidVrmlException g){
     System.out.println("Avatar " + name + " gesture string error: " + g );
     avdeck.drone_av_finished( name, false ); 
    return;
	}
   catch(Exception h){
     System.out.println("Avatar " + name + " gesture string error: " + h );
     avdeck.drone_av_finished( name, false ); 
    return;
	}
  for( int j = 0; j<gestures.length; j += 1 ){
    try {
gestures[j] = (EventInSFBool)nodes[0].getEventIn("gesture" + (j+1));
    }
  catch(NullPointerException f){
    System.out.println("Avatar " + name+" error getting gesture" + (j+1) +": " + f);
    avdeck.drone_av_finished( name, false );
    return;
	}
  catch(InvalidEventInException e){
   System.out.println("Avatar " + name+" error getting gesture" + (j+1) +": " + e);
    avdeck.drone_av_finished( name, false );
    return;
	}
   catch(InvalidVrmlException h){
    System.out.println("Avatar " + name+" error getting gesture" + (j+1) +": " + h);
    avdeck.drone_av_finished( name, false );
    return;
	}
   catch(Exception k){
    System.out.println("Avatar " + name+" error getting gesture" + (j+1) +": " + k);
    avdeck.drone_av_finished( name, false );
    return;
	}
 if(j==gestures.length-1){ bGest = true; }
   }
  } 
 EventOut EOfield2 = null;
 try{
 EOfield = null;
 bTest = false; 
 EOfield = nodes[0].getEventOut("inMotionBehaviour");
 EOfield2 = nodes[0].getEventOut("notInMotionBehaviour");
if((EOfield!=null&&EOfield.getType() == BaseField.SFString )&&(EOfield2!=null&&EOfield.getType() == BaseField.SFString )){
  bTest = true;
   }
 }
   catch (InvalidEventOutException d ){
   System.out.println("UserAvatar  doesn't have inMotionBehaviour or notInMotionBehaviour! " );
   }
   catch (NullPointerException f ){
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + f ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
   }
   	    catch (InvalidVrmlException j) {
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + j ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
	    }	
   catch (Exception ex) {
  System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + ex ); 
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  avdeck.drone_av_finished( name, false );
  avatar = null;
  return;
	    }	
 if(bTest){	  
   try {
		inMotionBehaviour = ((EventOutSFString)EOfield).getValue();
		notInMotionBehaviour = ((EventOutSFString)EOfield2).getValue();
		//walk = ((EventInSFBool)(nodes[0].getEventIn(inMotionBehaviour));
		stand = ((EventInSFBool)(nodes[0].getEventIn(notInMotionBehaviour)));
        System.out.println("Avatar " + name + " has humaniod behaviour!" ); 
	    }
        catch (InvalidEventOutException d ){
   System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + d );
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  stand = null;
  avdeck.drone_av_finished( name, false );
  return;
   }
   catch (NullPointerException f ){
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + f );
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  stand = null;
  avdeck.drone_av_finished( name, false );
  return;
   }
   	catch (InvalidVrmlException j) {
    System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + j ); 
   inMotionBehaviour = null;
  notInMotionBehaviour = null;
   stand = null;
  avdeck.drone_av_finished( name, false );
    return;
	    }	
   catch (Exception ex) {
  System.out.println("UserAvatar inMotionBehaviour or notInMotionBehaviour error: " + ex ); 
  inMotionBehaviour = null;
  notInMotionBehaviour = null;
  stand = null;
  avdeck.drone_av_finished( name, false );
  return;
	    }
  try{ 
EventInSFVec3f pt = (EventInSFVec3f)nodes[0].getEventIn("set_position");
EventInSFRotation rt = (EventInSFRotation)nodes[0].getEventIn("set_orientation");
}
catch(NullPointerException e){
    System.out.println(name + ": either eventIn set_position or set_orientation is missing for humanoid animation! " + e);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidEventInException f){
  System.out.println(name + ": either eventIn set_position or set_orientation is missing for humanoid animation!  "+ f);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidVrmlException g){
    System.out.println(name + ": either eventIn set_position or set_orientation is missing for humanoid animation! " + g);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(Exception h){
  System.out.println(name + ": either eventIn set_position or set_orientation is missing for humanoid animation!  "+ h);
    avdeck.drone_av_finished( name, false );
    return;
 }
  boolean bDMTest = false;
 try{
EOfield = null;
bTest = false;
EOfield = (EventOutSFBool)nodes[0].getEventOut("dmAvatarStudio");
if(EOfield!=null&&EOfield.getType() == BaseField.SFBool ){
bDMTest = true;
  }
 }
 catch(NullPointerException e){
   System.out.println( "UserAvatar dmAvatarStudio error: " + e);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(InvalidEventOutException f){
  System.out.println(name + ": is not a AvatarStudio Avatar");
 }
  catch(InvalidVrmlException j){
 System.out.println( "UserAvatar dmAvatarStudio error: " + j);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar dmAvatarStudio error: " + h);
    avdeck.drone_av_finished( name, false ); 
    avatar = null;
    return;
  } 
if(bDMTest){
	try {
eosfb = null;
eosfb = (EventOutSFBool)EOfield;
if(eosfb!=null){ 
if(eosfb.getValue()){ bdmStudioAvatar = true; }
   }
  }
catch(NullPointerException e){
  System.out.println( "UserAvatar dmAvatarStudio error: " + e);
    avdeck.drone_av_finished( name, false );
	avatar = null;
    return;
 }
catch(InvalidEventOutException f){
   System.out.println( "UserAvatar dmAvatarStudio error: " + f);
    avdeck.drone_av_finished( name, false );
   avatar = null;
    return;
 }
  catch(InvalidVrmlException j){
 System.out.println( "UserAvatar dmAvatarStudio error: " + j);
    avdeck.drone_av_finished( name, false );
    avatar = null;
    return;
 }
catch(Exception h){
System.out.println( "UserAvatar dmAvatarStudio error: " + h);
    avdeck.drone_av_finished( name, false );  
	avatar = null;
    return;
      }
     }	
  bHumanoid = true;
    }
if(bHumanoid||bGest){
try{ 
setProxy = (EventInSFInt32)nodes[0].getEventIn("set_proxy");
bAvatar = true;
}
catch(NullPointerException e){
    System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! " + e);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidEventInException f){
  System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! "+ f);
    avdeck.drone_av_finished( name, false );
    return;
 }
catch(InvalidVrmlException g){ 
System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! " + g);
     avdeck.drone_av_finished( name, false );
    return;
}
catch(Exception h){
System.out.println(name + ": eventIn set_proxy is either non existent or incorrect code! "+ h);
    avdeck.drone_av_finished( name, false );
    return;
   } 
 }
   }
	try {
  pos = (EventInSFVec3f)xform[0].getEventIn("translation");
	 rot = (EventInSFRotation)xform[0].getEventIn("rotation");
  pos2 = (EventOutSFVec3f)xform[0].getEventOut("translation");
	 rot2 = (EventOutSFRotation)xform[0].getEventOut("rotation");
  which = (EventInSFInt32)xform[0].getEventIn("whichChoice");
 if(lod.equals("NG_no_lod")){
   ((EventInMFFloat)xform[0].getEventIn("range")).setValue(fRange);
   } 
  }
  catch(InvalidEventOutException j) { 
System.out.println("Avatar " + name + " problem getting eventOuts from xform: " + j );  
 avdeck.drone_av_finished( name, false );
  return; 
  }
 catch(InvalidEventInException k) { 
System.out.println("Avatar " + name + " problem getting eventOuts from xform: " + k );  
 avdeck.drone_av_finished( name, false );
  return; 
  }
catch(InvalidNodeException d) {
System.out.println("Avatar " + name + " problem getting eventOuts from xform: "  + d );  
 avdeck.drone_av_finished( name, false );
  return;
  }
 catch (NullPointerException h) { 
System.out.println("Avatar " + name + " problem getting eventOuts from xform: " + h );
 avdeck.drone_av_finished( name, false );
  return; } 
 catch (InvalidVrmlException g) { 
System.out.println("Avatar " + name + " problem getting eventOuts from xform: " + g );
 avdeck.drone_av_finished( name, false );
  return; } 
 catch ( Exception j) { 
System.out.println("Avatar " + name + " problem getting eventOuts from xform: " + j );
 avdeck.drone_av_finished( name, false );
  return; } 

 anchor = xform[0];
 /**
     * New 11-12-05 If the avatar wants the chat display option this 
     * creates the AvatarTextTalk class to implement the option.
     */

if(yak!=null){ 
   texttalk = new AvatarTextTalk( postOffice, name2, bObject );
   texttalk.set_yakker(yak);
   System.out.println("Drone is text talker. ");
  }
 if(name.startsWith("_")){
 try {
  help3 = browser.createVrmlFromString( name_text );
 ((EventInMFNode)xform[0].getEventIn("addTXTChildren")).setValue(help3);
  }
 catch (InvalidVrmlException g) { 
System.out.println("Avatar drone Invalid VRML prob with name_text: " + g );
 avdeck.drone_av_finished( name, false );
  return; } 
catch (InvalidNodeException d) { System.out.println("Avatar " + name + "  Node prob with name_text: " + d );  
 avdeck.drone_av_finished( name, false );
  return; }
catch(InvalidBrowserException e){ System.out.println( "Avatar " + name + " Browser prob with name_text: " + e );
 avdeck.drone_av_finished( name, false );
 return;}
catch(NullPointerException f){System.out.println("Avatar " + name + " Null prob with name_text: " + f );
 avdeck.drone_av_finished( name, false );
  return; } 
catch( Exception g){System.out.println("Avatar " + name + " Exception prob with name_text: " + g );
 avdeck.drone_av_finished( name, false );
  return; } 
    }
  if(avdeck.checkIgnored(name2)==true){
  if(which!=null){ which.setValue(-1); bname_state = false; }
  }
if(bObject==true){ avdeck.handleObjNames( name, avUrl ) ;
 avdeck.drone_av_finished( name, true );
  return;
  }
else {
if(bHumanoid){
try {
browser.addRoute(anchor,"translation",avatar,"set_position");
browser.addRoute(anchor,"rotation",avatar,"set_orientation");
 }
 catch (InvalidEventOutException j) { 
System.out.println("Avatar " + name + " EventOut prob with creating ROUTES: " + j );  
 avdeck.drone_av_finished( name, false );
  return; }
 catch (InvalidEventInException k) { 
System.out.println("Avatar " + name + " EventOut prob with creating ROUTES: " + k );  
 avdeck.drone_av_finished( name, false );
  return; }
 catch (InvalidVrmlException g) { 
System.out.println("Avatar " + name + " Invalid VRML prob with creating ROUTES: " + g );
 avdeck.drone_av_finished( name, false );
  return; } 
catch (InvalidNodeException d) { 
System.out.println("Avatar " + name + "  Node prob with creating ROUTES: " + d );  
 avdeck.drone_av_finished( name, false );
  return; }
catch(InvalidBrowserException e){ 
System.out.println( "Avatar " + name + " Browser prob with creating ROUTES: " + e );
 avdeck.drone_av_finished( name, false );
 return;}
catch(NullPointerException f){
System.out.println("Avatar  " + name + " Null prob with creating ROUTES: " + f );
 avdeck.drone_av_finished( name, false );
  return; } 
 catch(Exception h){
System.out.println("Avatar  " + name + " Exception called creating ROUTES: " + h );
 avdeck.drone_av_finished( name, false );
  return; } 
    }
  avdeck.drone_av_finished( name, true );
   }
  }
 else { 
avdeck.drone_av_finished( name, false );
return; 
  }
 }
else
return;
} 
  
 public boolean getbObject(){
  return bObject;
 }

 public String getName(){
	return name;
    }

    /**
     * handles incoming real time messages addressed to this avatar.
     * it understands three different types :
     * <ul><li> SFVEC3F for position updates</li>
     *     <li> SFROT for orientation updates</li>
     *     <li> SFSTRING to trigger a behaviour</li></ul>
     * @param tag tag of the incoming message
     * @param data the field encoded in the message.
     */
    public void handleMessage( String tag, Field data ) throws ConsumerException {
	if(!tag.equals(name))
	    throw new ConsumerException(name+" is not equal to "+tag);
	try {  
	    switch(data.getType()){
	    case Field.SFVEC3F:
		pos.setValue(((SFVec3f)data).getValue());
        if(bAvatar){
        fCurrPos = ((SFVec3f)data).getValue(); 
        if(iUserNear1<100){ iUserNear1 += 1; } else { iUserNear1 = 0; } 
		if(iMove<100){ iMove += 1; } else { iMove = 0; }
        }
		break;
	    case Field.SFROTATION:
		rot.setValue(((SFRotation)data).getValue());
		//System.out.println("Av " + name + " handleMessage SFRotation: " + rflt[0] + " " + rflt[1] + " " + rflt[2] + " " + rflt[3] );
		if(iMove<100){ iMove += 1; } else { iMove = 0; }
		break;
		case Field.SFSTRING:
		if(avatar == null ) break;
        System.out.println("Avatar hM SFString1" );
		try {
		    EventInSFBool trig = (EventInSFBool)avatar.getEventIn(((SFString)data).getValue());
		    trig.setValue( true );
		}
		catch(Exception e){
		    System.err.println(name+" err setting "+((SFString)data).getValue()+"\n\t"+e);
		 }
		break;
		case Field.SFINT32:
		if(avatar == null ) break;
        if(iAv1==1){
		try {
		    int iGest = ((SFInt32)data).getValue();
            if(0<gestures.length){
            if(0<=iGest&&0<gestures.length){
		    gestures[iGest].setValue( true );
             }
           }
		}
		catch(Exception e){
		    System.err.println(name+" err setting "+((SFInt32)data).getValue()+"\n\t"+e);
		 }
        }
		break;
       default:
		throw new ConsumerException("Wrong field type for "+name);
	    }
	}
	catch(IllegalArgumentException e){
	    System.err.println("Error moving "+name+" "+e);
	}
    }

    public void update(){	
    }
   
    /**
    * New 11-12-05 gets values of position and rotation of the drone avatar and delivers
    * them to the UserAvatar so the user can beam to the drone
    */

   public void beamto( String target ){
if( target.equals(name) ){
   posflt = pos2.getValue(); rotflt = rot2.getValue();
   beam[0] = posflt[0]; beam[1] = posflt[1]; beam[2] = posflt[2];
   beam[3] = rotflt[0]; beam[4] = rotflt[1]; beam[5] = rotflt[2]; beam[6] = rotflt[3];
   avdeck.beamtocoord(beam);
 }
 } 

     public void userView( String target ){
  if(which!=null){
 if(bname_state){ which.setValue(-1); bname_state = false; }
 else { which.setValue(0); bname_state = true; }
  }
  avdeck.beamtoFinished();    
  }
  
  public void objectRoutes( String target ){
   float [] pflt = new float[3];
   float [] rflt = new float[4];
  if(bObject){
  //Browser browser = avdeck.getBrowser();
  Node PM = avdeck.getPrimeMover();
  if(bOR==true){
     avdeck.sharedObjectName( name, 1 );
     ((EventInSFVec3f)PM.getEventIn("set_start_position")).setValue( pos2.getValue() );
     pflt = pos2.getValue();
   rflt = rot2.getValue();
	 ((EventInSFRotation)PM.getEventIn("set_start_rotation")).setValue( rot2.getValue() );
   bOR = false;
   }
  else {
	avdeck.sharedObjectName( "", 0 );
   bOR = true;
      }
     }
   }

	 public boolean getbAvatar(){
     return bAvatar;
     }

     public void setUserPosition( float[] flt ){
     fUserPosition = flt;
  }

    public void checkPosition( float[] fUP, boolean bUPs  ){
  if(bDelete){
    if(iUserNear1!=iCheckPos||!bUPs){ 
	    float fX = Math.abs(fUP[0] - fCurrPos[0]);
		float fY = Math.abs(fUP[1] - fCurrPos[1]);
		float fZ = Math.abs(fUP[2] - fCurrPos[2]);
		if(fX<30&&fY<30&&fZ<30){ iAv1 = 1;
         }
        else { iAv1 = 0;
       }
     if(iAv1!=iAv2){ setProxy.setValue( iAv1 ); }
	    iCheckPos = iUserNear1; 
        }
     if(iMCt==2){ 
     if(iAv1==1){
     if(iMove==iMove2){
     if(stand!=null){ stand.setValue(true);} 
      iMove3 = iMove;  
      iMove2 = -1; 
         }
      else { 
      if(iMove3!=iMove){ 
      iMove2 = iMove; 
       }
         }
       }
       iMCt = 0;
           }
       iMCt +=1;
	   iAv2 = iAv1;
     }
   }
	 

    public void delete() throws RuntimeException {
bDelete = false;
if(setProxy!=null){ setProxy.setValue( 0 ); }
if(yak!=null){ texttalk.delete(); }
	Enumeration list = states.elements();
	while( list.hasMoreElements()){
	    postOffice.removeConsumer(((RealTimeConsumer)list.nextElement()).getName());
	}
	 list = vCtrlevents.elements();
 while( list.hasMoreElements()){
	    avdeck.removeCtrlEvent(((CtrlEvent)list.nextElement()).ctrlName());
} 
	states.removeAllElements();
	vCtrlevents.removeAllElements();
 if(WSC!=null){ WSC.delete(); }
 if(WSLC!=null){ WSLC.delete(); }
 yak = null; field = null; field2 = null; which = null; anchor = avatar = null; 
    }
 }