/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFImage field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFImage extends Field implements Serializable {
  public int width;
  public int height;
  public int components;
  public int [] pixels;
  public byte [] px;

  public SFImage( int w, int h, int c, int [] p ){
    setValue( w, h, c, p );
  }

  public SFImage( int w, int h, int c, byte[] p ){
    setValue2( w, h, c, p );
  }

  public SFImage( SFImage data ){
    width = data.width;
    height = data.height;
    components = data.components;
    pixels = data.pixels;
    px = data.px;
  }

  public SFImage(DataInputStream in) throws IOException {
    width = in.readInt();
    height = in.readInt();
    components = in.readInt();
    pixels = new int[width * height * components];
     int b = pixels.length;
     px = new byte[b];
     for ( int a = 0; a<b; a++ ){
   Integer foo = new Integer(pixels[a]);
    px [a] = foo.byteValue(); }
    in.readFully(px);
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(width);
    out.writeInt(height);
    out.writeInt(components);
    int b = pixels.length;
    for ( int a = 0; a<b; a++ ){
    out.writeInt(pixels[a]);}
	 out.write(px);
  }

  public byte getType() { return SFIMAGE; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFIMAGE )
      throw new MatrixInvalidFieldException("Data not SFImage field");
    setValue(((SFImage)data).width, ((SFImage)data).height, ((SFImage)data).components, ((SFImage)data).pixels);
  }

  public void setValue( int w, int h, int c, int [] p){
    width = w;
    height = h;
    components = c;
    pixels = p;
  }

  public void setValue2( int w, int h, int c, byte[] p){
    width = w;
    height = h;
    components = c;
	pixels = new int[width * height * components];
    px = p;
  }

  public int getWidth(){
    return width;
  }
  public int getHeight(){
    return height;
  }
  public int getComponents(){
    return components;
  }
  public int [] getPixels(){
    return pixels;
  }

  public byte[] getPx(){
    return px;
  }
}
