/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFInt32 field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFInt32 extends Field implements Serializable {
  public int value;

  public SFInt32( int val ){
    value = val;
  }

  public SFInt32( SFInt32 data ) {
    value = data.value;
  }

  public SFInt32(DataInputStream in) throws IOException {
    value = in.readInt();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeInt(value);
  }

  public byte getType() { return SFINT32; }
  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFINT32 )
      throw new MatrixInvalidFieldException("Data not SFInt32 field");
    value = ((SFInt32)data).value;
  }

  public void setValue( int value ){
    this.value = value;
  }

  public int getValue(){
    return value;
  }
}
