/* copyright (c) 1997 stephen f. white
 *  This file is part of DeepMatrix V1.
 *
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrix V1 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml;

import java.io.*;

/**
 * A class for SFRotation field
 * <BR>originally by Stephen F. White, additional features by Gerhard Reitmayr
 */
public class SFRotation extends Field implements Serializable {
  public float[] values = new float[4];

  public SFRotation(){}

  public SFRotation( SFRotation data ) {
    setValue( data.values );
  }

  public SFRotation(float axisX, float axisY, float axisZ, float angle) {
    values[0] = axisX;
    values[1] = axisY;
    values[2] = axisZ;
    values[3] = angle;
  }

  public SFRotation(float[] values) {
    setValue( values );
  }

  public SFRotation(DataInputStream in) throws IOException {
    values[0] = in.readFloat();
    values[1] = in.readFloat();
    values[2] = in.readFloat();
    values[3] = in.readFloat();
  }

  public void write(DataOutputStream out) throws IOException {
    out.writeFloat(values[0]);
    out.writeFloat(values[1]);
    out.writeFloat(values[2]);
    out.writeFloat(values[3]);
  }

  public byte getType() { return SFROTATION; }

  public void set( Field data ) throws MatrixInvalidFieldException {
    if( data.getType() != SFROTATION )
      throw new MatrixInvalidFieldException("Data not SFRotation field");
    setValue(((SFRotation)data).values);
  }

  public void setValue( float[] values ){
    this.values[0] = values[0];
    this.values[1] = values[1];
    this.values[2] = values[2];
    this.values[3] = values[3];
  }

  public float[] getValue(){
    return values;
  }
}
