/*
 *  This file is part of DeepMatrixIp9.
 *  Copyright (C) 2001 Geometrek
 *  DeepMatrix V1 is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  DeepMatrixIP9 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package matrix.vrml.eai;

import matrix.vrml.*;
import vrml.external.*;
import vrml.external.field.*;
import vrml.external.exception.*;

/**
 * Helper class to get and set EventOuts/Ins of the VRML EAI.
 */
public class EAITool {

    private EAITool(){
    }

    public static void setEventIn( EventIn event, Field value ) throws MatrixInvalidFieldException, IllegalArgumentException {
        switch( event.getType()){
            case FieldTypes.SFBOOL :
                if( value.getType() != value.SFBOOL )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFBool)event).setValue(((SFBool)value).value);
                break;
            //case FieldTypes.SFIMAGE :
             //   if( value.getType() != value.SFIMAGE )
             //       throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
             //   SFImage im = (SFImage)value;
             //   System.out.println("SFImage not yet implemented!");
                //((EventInSFImage)event).setValue(im.width, im.height, im.components, im.px );
             //   break;
            case FieldTypes.SFTIME :
                if( value.getType() != value.SFTIME )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFTime)event).setValue(((SFTime)value).time);
                break;
            case FieldTypes.SFCOLOR :
                if( value.getType() != value.SFCOLOR )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFColor)event).setValue(((SFColor)value).getValue());
                break;
            case FieldTypes.MFCOLOR :
                if( value.getType() != value.MFCOLOR )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFColor)event).setValue(((MFColor)value).getValue());
                break;
            case FieldTypes.SFFLOAT :
                if( value.getType() != value.SFFLOAT )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFFloat)event).setValue(((SFFloat)value).value);
                break;
            case FieldTypes.MFFLOAT :
                if( value.getType() != value.MFFLOAT )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFFloat)event).setValue(((MFFloat)value).values);
                break;
            case FieldTypes.SFINT32 :
                if( value.getType() != value.SFINT32 )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFInt32)event).setValue(((SFInt32)value).value);
                break;
            case FieldTypes.MFINT32 :
                if( value.getType() != value.MFINT32 )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFInt32)event).setValue(((MFInt32)value).values);
                break;
            case FieldTypes.SFROTATION :
                if( value.getType() != value.SFROTATION )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFRotation)event).setValue(((SFRotation)value).getValue());
                break;
            case FieldTypes.MFROTATION :
                if( value.getType() != value.MFROTATION )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFRotation)event).setValue(((MFRotation)value).getValue());
                break;
            case FieldTypes.SFSTRING :
                if( value.getType() != value.SFSTRING )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFString)event).setValue(((SFString)value).s);
                break;
            case FieldTypes.MFSTRING :
                if( value.getType() != value.MFSTRING )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFString)event).setValue(((MFString)value).strings);
                break;
            case FieldTypes.SFVEC2F :
                if( value.getType() != value.SFVEC2F )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFVec2f)event).setValue(((SFVec2f)value).getValue());
                break;
            case FieldTypes.MFVEC2F :
                if( value.getType() != value.MFVEC2F )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFVec2f)event).setValue(((MFVec2f)value).getValue());
                break;
            case FieldTypes.SFVEC3F :
                if( value.getType() != value.SFVEC3F )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInSFVec3f)event).setValue(((SFVec3f)value).getValue());
                break;
            case FieldTypes.MFVEC3F :
                if( value.getType() != value.MFVEC3F )
                    throw new MatrixInvalidFieldException("Field types not matching : "+event.getType()+"-"+value.getType());
                ((EventInMFVec3f)event).setValue(((MFVec3f)value).getValue());
                break;
            default:
                throw new MatrixInvalidFieldException("Not supported Field Type");
        }
    }

    public static Field makeField( EventOut event ) throws MatrixInvalidFieldException {
        switch( event.getType()){
            case FieldTypes.SFBOOL :
                return new SFBool(((EventOutSFBool)event).getValue());
            //case FieldTypes.SFIMAGE :
                //EventOutSFImage im = (EventOutSFImage)event;
             //return new SFImage(im.getWidth(), im.getHeight(), im.getNumComponents(), im.getPixels());
            case FieldTypes.SFTIME :
                return new SFTime(((EventOutSFTime)event).getValue());
            case FieldTypes.SFCOLOR :
                return new SFColor(((EventOutSFColor)event).getValue());
            case FieldTypes.MFCOLOR :
                return new MFColor(((EventOutMFColor)event).getValue());
            case FieldTypes.SFFLOAT :
                return new SFFloat(((EventOutSFFloat)event).getValue());
            case FieldTypes.MFFLOAT :
                return new MFFloat(((EventOutMFFloat)event).getValue());
            case FieldTypes.SFINT32 :
                return new SFInt32(((EventOutSFInt32)event).getValue());
            case FieldTypes.MFINT32 :
                return new MFInt32(((EventOutMFInt32)event).getValue());
            case FieldTypes.SFROTATION :
                return new SFRotation(((EventOutSFRotation)event).getValue());
            case FieldTypes.MFROTATION :
                return new MFRotation(((EventOutMFRotation)event).getValue());
            case FieldTypes.SFSTRING :
                return new SFString(((EventOutSFString)event).getValue());
            case FieldTypes.MFSTRING :
                return new MFString(((EventOutMFString)event).getValue());
            case FieldTypes.SFVEC2F :
                return new SFVec2f(((EventOutSFVec2f)event).getValue());
            case FieldTypes.MFVEC2F :
                return new MFVec2f(((EventOutMFVec2f)event).getValue());
            case FieldTypes.SFVEC3F :
                return new SFVec3f(((EventOutSFVec3f)event).getValue());
            case FieldTypes.MFVEC3F :
                return new MFVec3f(((EventOutMFVec3f)event).getValue());
            default:
                throw new MatrixInvalidFieldException("Not supported Field Type");
        }
    }
}
