/*
  Bot.java

  Copyright (C) 1998 Gerhard Reitmayr
*/

package tools;

import matrix.Message;
import matrix.deck.*;
import java.io.*;
import java.util.*;

/**
 * Bot Client for the Matrix System. used to test server loads etc.
 * uses special RealTimeConsumers to simulate users, etc. Reads its data from a resource file
 * with the following format :
 * <pre>hostname port name roomname
 * tools.name arg1 arg2 arg3 ...
 * # comments must start with '#' and be on a line by themselves
 * ...
 * </pre>
 * <tt>tools.name</tt> is the name of the class Bot should load. It must implement the BotConsumer
 * interface. The rest of the line are space separated args ( max 10 ) to the BotConsumer, given it via setData.
 *
 * @version 0.5
 * @author Gerhard Reitmayr
 * @see tools.BotConsumer
 */
public class Bot2 implements NetworkListener, Runnable {

  // Matrix stuff
  private Backend net;
  private Vector consumers = new Vector();
  private int num, port, a = 0, d = 0;
  private long pause;
  private String name;
  private String password;
  private Class consumerClass;
  private Object consumerObject;
  private  String classname, host, room, args2[];
  private  Vector argvec = new Vector();
   private Thread runner;
  private Heart2 heart;
  public static void main( String[] args ) throws Exception {
    Bot2 me;
 System.err.println(args[0]);
    if( args.length < 1 ) {
      System.err.println("Usage : java tools.Bot resourcefile");
      return;
    }
    try{     
      me = new Bot2( args[0] );
      while( new FileInputStream(args[0]).available() == 0){
        try{
            Thread.sleep(1000);
        }
        catch(InterruptedException e){}
      }
    }
    catch(Exception e){
      System.err.println("Error running Bot "+e);
      System.exit(1);
    }
  }

  public Bot2( String file ) throws Exception {
    net = new Backend( this, 1 );

    // read resource file
    BufferedReader fin = new BufferedReader(new FileReader(file));

    // must be arguments for connecting
    String line = fin.readLine();
    while( line.startsWith("#") )
      line = fin.readLine();
    StringTokenizer token = new StringTokenizer( line, " " );
    host = token.nextToken();
    port = Integer.parseInt( token.nextToken());
    password = token.nextToken();
    name = token.nextToken();
    room = token.nextToken();
    room = room.replace('_',' ');
    num =  Integer.parseInt( token.nextToken());
    pause = Long.valueOf( token.nextToken() ).longValue(); 
    System.out.println("Connecting to "+host+":"+port+" as "+name+" using password: " + password + " in room "+room+"."+num+" bots");

    // read the rest of the lines
    line = fin.readLine();
    while( line != null ){
      if( line.length() == 0 || line.startsWith("#")){
        line = fin.readLine();
        continue;
      }
      token = new StringTokenizer( line, " " );
      consumerClass = Class.forName( token.nextToken());
     
      argvec.removeAllElements();
      while( token.hasMoreTokens()){
        argvec.addElement( token.nextToken());
      }
      args2 = new String[ argvec.size()];
      argvec.copyInto( args2 );
    for(int b = 0; b< args2.length; b++){
      System.out.println( args2[b] );}
      line = fin.readLine();
    }
    fin.close();
    this.start();
  }
  public void start(){

    runner = new Thread( this, name );
    runner.start();

    System.out.println("MultiBots of "+name+" started");
  }

  public void stop(){
    runner = null;
  }

  public void run(){ 
    while( Thread.currentThread() == runner ){
int c = 0;
if(  a<num ){ 
if(0<a){args2[0] = ("_"+name+(a+1));}
else { args2[0] = ("_"+name );}
System.out.println(args2[0]);
try {
   consumerObject = consumerClass.newInstance();}
 catch(IllegalAccessException e){}
 catch(InstantiationException e){} 
 consumers.addElement( consumerObject );
try { 
   ((BotConsumer)consumers.elementAt(a)).setData( net, args2 ); }
  catch(ConsumerException e ){ System.out.println("consumer not found");}
if(a==0){ try {
    net.connect( host, port, args2[0].substring(1), password, room, "BOT" );}
    catch(IOException e){System.out.println("No connection");}
     handleHeartStart();     
      }  
    Enumeration list = consumers.elements();
    // start all BotConsumers
((BotConsumer)consumers.elementAt(a)).start();
 a +=1; 
 try {
 runner.sleep ( (pause) );}
 catch(InterruptedException e){} }
  else { this.stop();}}
    }

  public void handleLOGOUT() {
    System.out.println("LOGOUT received ... shutting down");
    System.exit(0);
  }

    private void handleHeartStart(){
   heart = new Heart2( this, 5000 );
  }

  public void handleOBJADD( String name, String url ){
    System.out.println("Avatar "+name+"("+url+") added");
    net.addConsumer( new ObjectListener( name+a, false));
  }

  public void handleOBJDEL( String name ){
    net.removeConsumer( name );
    System.out.println("Avatar "+name+" removed");
  }


  public void handleMSG( String from, String line ){
    System.out.println(from+" : "+line);
  }



  public void heartBeat() throws IOException {
  net.sendHeartBeat( name );	
     }

  public void handleWHOIS( String[] users ){
    System.out.print("Userlist : ");
    for( int i = 0; i < users.length; i++)
      System.out.print(users[i]+" ");
    System.out.println("");
  }

  public void handleERROR( String from, String line ){
    System.out.println(from+" : "+line);
  }
  
  public void handleCTRLEV( String name, String [] lock ){
  }

  public void handleOBJIGN( String name, String tf ){
  }

  public void handleNEWRM( String new_room ){
  }
  
  public void handleROOM_A( String name ) throws InterruptedException {
  }
  
  public void handleROOM_B( String name, String url ) throws InterruptedException {
  }

  public void handleNetFailure( String arg ){
  System.out.println( "handleNetFailure: " + arg );
  }

  public void handleOBJIS( String[] objects ){
  }

  public void handleWWW( Message msg ){
  }

  public void handleADMIN( int arg ){
   }

   public void handlePOLL( float fsto ){
   }

  public void handleNavChange( String [] hNC ){
   }

  public void handleSpeedChange( float ft ){
   }

  public void close(){
    // stop all BotConsumers
    Enumeration list = consumers.elements();
    while( list.hasMoreElements())
      ((BotConsumer)list.nextElement()).stop();
    net.disconnect( "false" );
    net.close();
  }
}
